/*! \file main.c
 *  \brief Main
 *         3DS main prg.
 *
 *  \version 0.1
 *
 *  3DS main src
 *
 *   Pituka - Dingoo Port
 *  (c) Copyright 2016 David Colmenero (aka D_Skywalk)
 */

#include <string.h>
#include <3ds.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include "../global.h"
#include "game_common.h"
#include "../abadia/interface.h"

//#include "DSKCPC_zip_bin.h"
#include "SNACPC_sna_bin.h"

//400x240
//320x240

//#define DOUBLE_VIDEO_BUFFER

/*** EMU Globals ***/
void *pix = NULL;
extern Bitu8 keyboard_matrix[16];
extern Bitu8 bit_values[8];

extern unsigned char * pbGPBuffer;

char current_path [1024] = "/CPCROMS/";

int screen_bps;
unsigned short *screen_base;

WiituKa_Status WiiStatus = {0, 0, 0, 0, 0, 0, 0, 1, 0};
xmlWiiCFG WiitukaXML;
t_WiiRom globalRom;

char extension[5];
bool emuDone = false;
/* 3DS TIMER */

static struct timeval start;

/* spool */
char spool_cad[20];
bool spool;
bool spool_act;

Pituka_SpoolKeys spoolkeys;
unsigned char reiniciado;

unsigned int GetTicks(void) {

    struct timeval now;
    gettimeofday(&now, NULL);
    return (now.tv_sec - start.tv_sec) * 1000 + (now.tv_usec - start.tv_usec) / 1000;

    //return (unsigned int) svcGetSystemTick();
}

static aptHookCookie aptCookie;
static void aptCallback(APT_HookType hook, void* param) {
    switch (hook) {
    case APTHOOK_ONSLEEP:
    case APTHOOK_ONSUSPEND:
        StopSound(true);
        game_save();
        break;
    case APTHOOK_ONWAKEUP:
    case APTHOOK_ONRESTORE:
        StopSound(false);
    default:
        break;
    }
}

void init_3ds(void) {
    aptInit();
    cfguInit();
    hidInit();

    aptHook(&aptCookie, aptCallback, NULL);

    // init screen 16bits 565 screen buffer data
    gfxInit(GSP_RGB565_OES, GSP_RGB565_OES, false);

    gfxSetDoubleBuffering(GFX_TOP, true);
    gfxSetDoubleBuffering(GFX_BOTTOM, false);
    consoleInit(GFX_BOTTOM, NULL);
    gfxSet3D(false);

    gettimeofday(&start, NULL);
    CFGU_GetSystemLanguage(&WiitukaXML.language);
    // si no es castellano (35) es cualquier otro, quitamos 1 bit para indicarlo.
    // forzamos interfaz en inglés (34)
    if(WiitukaXML.language != CFG_LANGUAGE_ES)
        WiitukaXML.language = (CFG_LANGUAGE_ES ^ 1);

    pix = (void *)linearAlloc(4 * WRES_3DS * HRES_3DS);

    pbGPBuffer = (unsigned char*) malloc(128 * 1024); //need by unzip lib and DSK
    InitGFX();

}

void close_3ds(void) {
    FreeGFX();

    if (pbGPBuffer != NULL) {
        free(pbGPBuffer);
        pbGPBuffer = NULL;
    }

    doCleanUp();
    printf("cleared emu!\n");

    if (pix != NULL) {
        linearFree(pix);
    }

    pix = NULL;
    printf("clear screen!\n");

    aptUnhook(&aptCookie);
    printf("clear hook!\n");
    gfxExit();
    hidExit();
    aptExit();
}

void SoundUpdate(void);
bool create3DSPath(char *path);

inline void UpdateScreen(void) {
    u32 * src;
    u32 * dest;

    dest = (u32*) gfxGetFramebuffer(GFX_TOP, GFX_LEFT, NULL, NULL);
    src = (u32*)(pix + 0);

    register int x = HRES_3DS * (WRES_3DS / 2);

    while (x--)
        *(dest++) = (Bitu32) * (src++);

    // Flush and swap framebuffers
    gfxFlushBuffers();
    gfxSwapBuffers();
    gspWaitForVBlank();
}

void (*run_func) (void);

int main() {
    WiiStatus.LoadDISK = true;
    globalRom.location = SU_SD;
    run_func = game_options;

    init_3ds();

    ShowSplash();

    if (!cpc_init())
        printf("Emu Init - Failed!");

    game_init();
    create3DSPath(EMU_PATH);

    while (aptMainLoop()) {
        (*run_func)();
        gfxFlushBuffers();

#ifndef _CIA
        if (emuDone)
            break;
#endif
    }

    game_save();
    StopSound(true);
    close_3ds();

    return 0;
}

inline void poll_input(void) {
    register int i;
    hidScanInput();

    u32 kDown = hidKeysDown();
    if (kDown) {
        if (kDown & (KEY_L|KEY_R))
            run_func = game_options;
        i = 12;
        while (i--)
            if ((kDown & BIT(i)) && (translate3DS[i] != 0xff))
                keyboard_matrix[translate3DS[i] >> 4] &= ~bit_values[translate3DS[i] & 7]; // key is being held down
    }

    u32 kUp = hidKeysUp();
    if (kUp) {
        i = 12;
        while (i--)
            if ((kUp & BIT(i)) && (translate3DS[i] != 0xff))
                keyboard_matrix[translate3DS[i] >> 4] |= bit_values[translate3DS[i] & 7]; // key is being held down

    }
}

int load_sna(void) {
    bool result = false;
    if ((result = snapshot_load_buffered((void*) SNACPC_sna_bin, SNACPC_sna_bin_size))) {
        printf(" LOAD BUFFER SNA: BAD\n");
    }
    return result;
}

void main_process_pause(int val) {}
extern Button buttons[MAX_BTN];

bool create3DSPath(char *path) {
    int error = 0;
    error = chdir(path);
    if (error) { //NOT FOUND?
        if (mkdir(path, S_IREAD | S_IWRITE) == -1) { //CREATE IT!
            printf("Err CreateDirs: mkdir: %s\n", path);
            return false;
        }

        error = chdir(SAVE_PATH);
        if (error) {
            if (mkdir(SAVE_PATH, S_IREAD | S_IWRITE) == -1) {
                printf("Err CreateDirs: mkdir: %s\n", SAVE_PATH);
                return false;
            }
        }
    } else {
        // check save exists
        FILE *file = fopen(SAVE_FILE, "r");
        if (file) {
            buttons[BTN_LOAD].status |= BTN_ENABLE;
            fclose(file);
        }
    }
    return true;
}
