/*! \file interface.c
 *  \brief Gui Driver.
 *         3DS Library.
 *
 *  \version 0.9
 *
 *  3DS Interface/Gui driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2016 David Colmenero (aka D_Skywalk)
 */

#include <3ds.h>
#include "../global.h"

#include "imgu.h"
#include "interface.h"

#include "splash_png.h"
#include "info_eng_png.h"
#include "info_esp_png.h"
#include "save_eng_png.h"
#include "save_esp_png.h"
#include "leters_png.h"
#include "logros_png.h"
#include "options_png.h"
#include "options_bottom_png.h"
#include "options_btn_png.h"
#include "options_btn_game_png.h"
#include "title_png.h"

extern xmlWiiCFG WiitukaXML;
t_img info, save, leters, logros, options, options_bottom, options_btn, options_btn_game;

Button buttons[MAX_BTN] = {
    {122,120,190,178, BTN_ENABLE }, // NEW
    { 48,120,122,178, BTN_DISABLE}, // SAVE
    {196,120,262,178, BTN_DISABLE}, // LOAD
    {166, 64,286, 99, BTN_ENABLE},  // COLOR
    { 30, 64,150, 99, BTN_ENABLE},  // LANG
    {  0,211, 80,234, BTN_DISABLE}, // BACK
    {228,211,310,234, BTN_DISABLE}, // OK
};

bool InitGFX(void) {
    InitImg(&leters);
    if (!LoadIMG(&leters, leters_png, leters_png_size))
        return false;

    InitImg(&logros);
    if (!LoadIMG(&logros, logros_png, logros_png_size))
        return false;

    InitImg(&options);
    if (!LoadIMG(&options, options_png, options_png_size))
        return false;
    InitImg(&options_bottom);
    if (!LoadIMG(&options_bottom, options_bottom_png, options_bottom_png_size))
        return false;

    InitImg(&options_btn);
    if (!LoadIMG(&options_btn, options_btn_png, options_btn_png_size))
        return false;

    InitImg(&options_btn_game);
    if (!LoadIMG(&options_btn_game, options_btn_game_png, options_btn_game_png_size))
        return false;

    InitImg(&info);
    if(WiitukaXML.language == CFG_LANGUAGE_ES) {
        if (!LoadIMG(&info, info_esp_png, info_esp_png_size))
            return false;
        if (!LoadIMG(&save, save_esp_png, save_esp_png_size))
            return false;

    }else{
        if (!LoadIMG(&info, info_eng_png, info_eng_png_size))
            return false;
        if (!LoadIMG(&save, save_eng_png, save_eng_png_size))
            return false;
    }
    return true;
}

void FreeGFX(void) {
    FreeImg(&leters);
    FreeImg(&logros);
    FreeImg(&options);
    FreeImg(&options_bottom);
    FreeImg(&options_btn);
    FreeImg(&options_btn_game);
    FreeImg(&info);
    FreeImg(&save);
}

void ShowSplash(void) {
    t_img splash, title;
    InitImg(&splash);
    InitImg(&title);

    if (!LoadIMG(&splash, splash_png, splash_png_size))
        return;
    if (!LoadIMG(&title, title_png, title_png_size))
        return;

    DrawIMG(GFX_TOP, &splash);
    svcSleepThread(3000000000LL); //3 secs

    DrawIMG(GFX_TOP, &title);

    FreeImg(&splash);
    FreeImg(&title);
}

void ShowSave(void) {
    DrawIMG(GFX_BOTTOM, &save);
    buttons[BTN_LOAD].status |= BTN_ENABLE;
}

void ShowInfo(void) {
    DrawIMG(GFX_BOTTOM, &info);
}

void ShowOptions(void) {
    DrawIMG(GFX_BOTTOM, &options);
}


#define LOGRO_W 40
#define LOGRO_H 38
#define LOGRO_X 200
#define LOGRO_Y 77
void ShowLogro(int value) {
    Goo_Rect org = {LOGRO_W * value, 0, LOGRO_H, LOGRO_W};
    Goo_Rect dest = {LOGRO_X + (LOGRO_W * (value % 3)), LOGRO_Y + (LOGRO_H * (value / 3)), 0, 0};

    DrawRect(GFX_BOTTOM, &logros, &org, &dest);
}

#define NUMBER_W 14
#define SCORE_X 21
#define SCORE_Y 452
void ShowStr(char * string) {
    char chr;
    int value;
    for (int len = 0; string[len] != '\0'; len++) {
        chr = string[len];
        switch (chr) {
            case ' ':
                value = 30; break;
            case '!':
                value = 29; break;
            case ',':
                value = 28; break;
            case '.':
                value = 27; break;
            case '&':
                value = 26; break;
            default:
                value = ((int) chr) - 'A';
                if(value < 0 || value > 26)
                    continue;

        }
        //printf("%c: %i (%i)\n", string[len], (int) string[len], value);

        Goo_Rect org = {NUMBER_W * value, 0, 16, NUMBER_W};
        Goo_Rect dest = {SCORE_X + NUMBER_W * len, SCORE_Y, 0, 0};

        DrawGFX(GFX_BOTTOM, &leters, &org, &dest);

    }
    FlushGFX();
}

void ShowQR (void) {
    Goo_Rect org = {512, 0, 36, 36};
    Goo_Rect dest = {0, 0, 0, 0};
    DrawGFX(GFX_BOTTOM, &options_btn, &org, &dest);
    dest.x = 284; org.x = 548;
    DrawGFX(GFX_BOTTOM, &options_btn, &org, &dest);
}

void ShowButtons (void) {
    t_img * img = NULL;
    Goo_Rect org = {0, 0, 0, 0};
    Goo_Rect dest = {0, 0, 0, 0};
    register int i = MAX_BTN - 1;

    while(i--){
        Button *btn = &buttons[i];
        if(!(btn->status & BTN_ENABLE)){
            if(btn->status & BTN_CLEAN){
                //drawgfx no soporta coger parte de una imagen el ancho debe ser completo
                org.w = 240; org.h = 82;
                DrawGFX(GFX_BOTTOM, &options, &org, &org);
            }
            continue;
        }

        dest.x = btn->x; dest.y = btn->y;

        switch (i) {
            case BTN_OK:
                org.x = 82; org.y = 0; org.w = 32; org.h = 82;
                img = &options_bottom;
                break;
            case BTN_BACK:
                org.x = org.y = 0; org.w = 32; org.h = 82;
                img = &options_bottom;
                break;

            case BTN_SAVE:
                org.x = 0; org.y = 0; org.w = 74; org.h = 74;
                if(btn->status & BTN_PUSHED)
                    org.x += 222;
                img = &options_btn_game;
                break;
            case BTN_NEW:
                org.x = 74; org.y = 0; org.w = 74; org.h = 74;
                if(btn->status & BTN_PUSHED)
                    org.x += 222;
                img = &options_btn_game;
                break;
            case BTN_LOAD:
                org.x = 148; org.y = 0; org.w = 74; org.h = 74;
                if(btn->status & BTN_PUSHED)
                    org.x += 222;
                img = &options_btn_game;
                break;

            case BTN_LANG:
                org.x = 0; org.y = 0; org.w = 36; org.h = 128;
                if(WiitukaXML.language == CFG_LANGUAGE_ES)
                    org.x += 128;
                img = &options_btn;
                break;
            case BTN_COLOR:
                org.x = 256; org.y = 0; org.w = 36; org.h = 128;
                if(!WiitukaXML.scrtube)
                    org.x += 128;
                img = &options_btn;
                break;
        }
        DrawGFX(GFX_BOTTOM, img, &org, &dest);
    }
    FlushGFX();
}

