/*! \file interface.c
 *  \brief Gui Driver.
 *         3DS Library.
 *
 *  \version 0.9
 *
 *  3DS Interface/Gui driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2016 David Colmenero (aka D_Skywalk)
 */

#include <3ds.h>
#include "../global.h"
#include "../3ds/game_common.h"

#include "logros.h"
#include "interface.h"

extern u8 * pbRAM;
extern u16 * fullday;
#define MSK_SCORE 0b111111111 // 511

unsigned int last_logros;
unsigned int remove_func;

void func_bit_check(t_logro * logro);
void func_compare(t_logro * logro);
void func_scouty(t_logro * logro);
void func_obbey(t_logro * logro);

#define PTR_SCRIPT_DAY                  0x2D80
#define PTR_SCRIPT_HOUR                 0x2D81
/*
 0x2d80 número de día actual (del 1 al 7)
 0x2d81 momento del día actual
    0 = noche
    1 = prima
    2 = tercia
    3 = sexta
    4 = nona
    5 = visperas
    6 = completas
*/

#define PTR_SCRIPT_CAM_FOLLOW           0x2D88
#define PTR_SCRIPT_MIRROR_STATUS        0x2D8C
#define PTR_SCREEN_NUMBER               0x2DBD
#define PTR_SCORE_BONUS                 0x2DBE
#define PTR_INV_GUILLERMO               0x2DEF
#define PTR_LMP_ADSO                    0x2DF3
#define PTR_INV_ADSO                    0x2DF6 //unused atm
#define PTR_SCRIPT_WILLIAM_PAPYRE       0x3C90
#define PTR_SCRIPT_MALAQUIAS_STATUS     0x3CA2
#define PTR_SCRIPT_BERNARDO_PERGAMINO   0x3CE8

//NIGHT: 2DBE & 0 = ABADIA IZQ DE NOCHE (O 0x2d80 + 0x2d81 + 0x3c99 y decir no)
t_logro logro_night = {
    POS_NIGHT, "NIGHT",
    MSK_FILTER_NIGHT_HOUR, MSK_FILTER_NIGHT_DAY,
    func_bit_check,
    PTR_SCORE_BONUS,
    0x01, //MSK_NIGHT
    NULL,
};
// LIGHT: 2DF6 & 7 = LAMPARA ADSO
t_logro logro_light = {
    POS_LIGHT, "LIGHT",
    MSK_FILTER_LIGHT_HOUR, MSK_FILTER_LIGHT_DAY,
    func_bit_check,
    PTR_LMP_ADSO,
    BIT(7),
    NULL,
};
// RIGHT: 2DBE & 1 = LLAVE 3 (Pasadizo), adso coge la llave del Pasadizo, gana bonus
t_logro logro_right = {
    POS_RIGHT, "RIGHT",
    MSK_FILTER_RIGHT_HOUR, MSK_FILTER_RIGHT_DAY,
    func_bit_check,
    PTR_SCORE_BONUS,
    BIT(1),
    NULL,
};

// SCOUTY: (0x2DBD nº de pantalla)
t_logro logro_scouty = {
    POS_SCOUTY, "SCOUTY",
    MSK_FILTER_SCOUTY_HOUR, MSK_FILTER_SCOUTY_DAY,
    func_scouty,
    PTR_SCREEN_NUMBER,
    -1,
    NULL,
};
// LIBRARY: 0x2DBE & 5 (adso con lampara en biblio)
t_logro logro_library = {
    POS_LIBRARY, "LIBRARY",
    MSK_FILTER_LIBRARY_HOUR, MSK_FILTER_LIBRARY_DAY,
    func_bit_check,
    PTR_SCORE_BONUS,
    BIT(5),
    NULL,
};
// OBEY: DAR PERGAMINO AL ABAD (0x3c94 == 0/1), val init: 0
t_logro logro_obey = {
    POS_OBEY, "OBEY",
    MSK_FILTER_OBEY_HOUR, MSK_FILTER_OBEY_DAY,
    func_obbey,
    PTR_SCRIPT_BERNARDO_PERGAMINO,
    0x07, // VALUE OBBEY
    NULL,
};

// DECEASE:  MUERE MALAQUIAS (0x3ca2 == 2), val init: 0
t_logro logro_decease = {
    POS_DECEASE, "DECEASE",
    MSK_FILTER_DECEASE_HOUR, MSK_FILTER_DECEASE_DAY,
    func_bit_check,
    PTR_SCRIPT_MALAQUIAS_STATUS,
    0x02, // MSK_DEAD
    NULL,
};
// PAPYRE: COGER PERGAMINO (0x3c90, si lo coges vale 0)
t_logro logro_papyre = {
    POS_PAPYRE, "PAPYRE",
    MSK_FILTER_PAPYRE_HOUR, MSK_FILTER_PAPYRE_DAY,
    func_bit_check,
    PTR_INV_GUILLERMO,
    0x10, // MSK_PAPIRE_OBJ
    NULL,
};
// FINALE: Q/R (0x2d8c == 0)
t_logro logro_finale = {
    POS_FINALE, "FINALE",
    MSK_FILTER_FINALE_HOUR, MSK_FILTER_FINALE_DAY,
    func_compare,
    PTR_SCRIPT_MIRROR_STATUS,
    0, NULL,
};


t_logro * base_logros[MAX_LOGROS] = {
        &logro_decease, &logro_scouty, &logro_night,
        &logro_papyre, &logro_library, &logro_light,
        &logro_finale, &logro_obey, &logro_right,
    };

int current_logros;
u8 *ptrDay, *ptrHour, *ptrCam;
u8 last_pantalla = -1;

/**
 * nos eliminamos del loop añadiendonos a la remove list
 * @param val logro a eliminar
 */
void remove_me(int value){
    ShowLogro(value);
    remove_func |= (1 << value);
}

void logros_print(void) {
    printf("M[%i] S[%i] P[%i] ES[%i] LOGROS:%i\n", *logro_decease.ptr, *logro_scouty.ptr, *logro_papyre.ptr, *logro_finale.ptr, current_logros);
    printf("D[%i:%i] B[%i] A[%i] CAM[%i] F%04x\n", *ptrDay, *ptrHour, *logro_night.ptr, *logro_light.ptr, *ptrCam, *fullday);
}


/**
    recrea el array de funciones
    inicializa remove_func y current_logros
 */
void logros_set(void) {
    // XOR 10001000 ^ 10000000 = 00001000
    last_logros ^= remove_func;
    remove_func = 0;
    current_logros = 0;

    register int i = MAX_LOGROS;
    while(i--){
        if(! (last_logros & BIT(i)) )
            continue;

        if(! (base_logros[i]->hour & (1<<*ptrHour)) )
            continue;

        if(! (base_logros[i]->day & (1<<*ptrDay)) )
            continue;

        logros_ptr[current_logros] = base_logros[i];
        current_logros++;
    }
}

void func_bit_check(t_logro * logro) {
    if(! ((*logro->ptr) & logro->val) )
        return;
    remove_me(logro->pos);
}

void func_compare(t_logro * logro) {
    if(*logro->ptr)
        return;
    remove_me(logro->pos);
}

int scouty_log = 0;
void func_scouty(t_logro * logro) {
    if(logro->val == (*logro->ptr))
        return;

    logro->val = (*logro->ptr);
    scouty_log ++;


    if(scouty_log > 100)
        remove_me(logro->pos);
}

void func_obbey(t_logro * logro) {
    if((*logro->ptr) != logro->val)
        return;
    remove_me(logro->pos);
}

/*

 ***************
 3D1D, datos del script
 2DEF, objetos de guillermo
 2DF6, objetos de adso

marcador lampara:
0x3c8b

 Sumando puedes obtener un falso positivo (0+0+1+ 0xFFFFFFFF = 0), hay que hacer value = visit[0] | visit[1] | visit[2] | visit[3];

 Si quieres hacerlo rápido sin ASM, haz una tabla de máscaras
 mask[32]= { 0xFFFFFFFE, 0xFFFFFFFD ... 0x7FFFFFFF }

 Y el siguiente código:
 id_high = (id>>5) & 3;
 Id_low = id & 31;
 Visit[id_high] &= mask[id_low];

*/

void logros_draw(void) {
    register int i = MAX_LOGROS;
    while(i--)
        if(! (last_logros & BIT(i)) )
            ShowLogro(i);
}

/**
    carga el record y resetea remove_func
 */
void logros_load(char * filepath) {
    last_logros = MSK_SCORE;
    remove_func = 0;
    current_logros = 0;
    FILE *file = fopen(filepath, "r");
    if (file == NULL)
        return;

    if (fread(&last_logros, sizeof(unsigned int), 1, file) == 0) {
        printf("Error reading game data: %s.\n", filepath);
        fclose(file);
        return; // exit if loading of data chunck failed
    }

    fclose(file);
}

bool logros_save(char * filepath) {
    FILE *file = fopen(filepath, "w");
    if (file == NULL) {
        printf("Error saving game data: %s\n", filepath);
        return false;
    }

    fwrite(&last_logros, sizeof(unsigned int), 1, file);
    fclose(file);
    return true;
}

void logros_init(void) {
    int i = MAX_LOGROS;

    ptrDay = &pbRAM[PTR_SCRIPT_DAY];
    ptrHour = &pbRAM[PTR_SCRIPT_HOUR];
    ptrCam = &pbRAM[PTR_SCRIPT_CAM_FOLLOW];

    logros_set();

    while(i--)
        base_logros[i]->ptr = &pbRAM[base_logros[i]->addr];
}

