/*! \file rick.c
 *  \brief rick game driver.
 *
 *  \version 1.0
 *
 *  In Game driver
 *
 *   Pituka - 3DS Port
 *  (c) Copyright 2009 David Colmenero (aka D_Skywalk)
 */

#include <string.h>
#include <3ds.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include "../global.h"
#include "../3ds/game_common.h"
#include "interface.h"

extern u8 * pbRAM;

u8* ptrLife;
unsigned int last_score = 0;
bool need_save = false;

u8 translate3DS[32] = { //rick pad cfg
    0x57, // A
    0x57, // B
    0xff, // SELECT
    0xff, // START
    0x93, // DRIGHT
    0x92, // DLEFT
    0x90, // DUP
    0x91, // DDOWN
    0xff, // R
    0xff, // L
    0xff, // X
    0x57, // Y
    0xff, 0xff,
    0xff, // ZL
    0xff, // ZR
    0xff, 0xff, 0xff, 0xff,
    0xff, // TOUCH
    0xff, 0xff, 0xff,

    0xff, // CSTICK_RIGHT
    0xff, // CSTICK_LEFT
    0xff, // CSTICK_UP
    0xff, // CSTICK_DOWN

    0x93, // CPAD_RIGHT
    0x92, // CPAD_LEFT
    0x90, // CPAD_UP
    0x91, // CPAD_DOWN
};

#define SCORE_RESET_VALUE 93000 // first value on game loaded

#define SAVE_FILE SAVE_PATH "/RICK.BIN"

void game_showscore(void) {
    char str[10];
    sprintf(str, "%06u", last_score);
    ShowNumbers(str);
}

bool game_load(void) {
    FILE *file = fopen(SAVE_FILE, "r");
    if (file == NULL) {
        last_score = 0;
        return true;
    }

    if (fread(&last_score, sizeof(unsigned int), 1, file) == 0) {
        printf("Error reading game data: %s.\n", SAVE_FILE);
        fclose(file);
        return false; // exit if loading of data chunck failed
    }

    fclose(file);

    game_showscore();
    return true;
}

bool game_init(void) {
    ptrLife = &pbRAM[0x49e2];
    create3DSPath(EMU_PATH);
    game_load();

    need_save = false;
    return true;

}

inline void game_run(void) {
    if (*ptrLife)
        return;

    unsigned int score = pbRAM[0x4a82] * 10 + pbRAM[0x4a81] * 100 + pbRAM[0x4a80] * 1000 + pbRAM[0x4a7f] * 10000 + pbRAM[0x4a7e] * 100000;
    if (score == SCORE_RESET_VALUE)
        return;

    if (last_score < score) {
        last_score = score;
        need_save = true;
        game_showscore();
    }
}

bool game_save(void) {
    if (!need_save)
        return true;

    FILE *file = fopen(SAVE_FILE, "w");
    if (file == NULL) {
        printf("Error saving game data: %s\n", SAVE_FILE);
        return false;
    }

    fwrite(&last_score, sizeof(unsigned int), 1, file);
    fclose(file);
    need_save = false;
    return true;
}
